@@ -12,7 +12,7 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  *
- * Copyright (C) 2015-present, TBOOX Open Source Group.
+ * Copyright (C) 2015-present, Xmake Open Source Community.
  *
  * @author      ruki
  * @file        md5.c
@@ -56,12 +56,11 @@ tb_int_t xm_hash_md5(lua_State* lua)
         tb_md5_make(data, size, buffer, sizeof(buffer));
 
         // make md5 string
-        tb_size_t i = 0;
-        tb_char_t s[256] = {0};
-        for (i = 0; i < 16; ++i) tb_snprintf(s + (i << 1), 3, "%02x", buffer[i]);
+        tb_char_t s[256];
+        tb_size_t n = xm_hash_make_cstr(s, buffer, 16);
 
         // save result
-        lua_pushstring(lua, s);
+        lua_pushlstring(lua, s, n);
         return 1;
     }
 
@@ -109,14 +108,11 @@ tb_int_t xm_hash_md5(lua_State* lua)
             tb_md5_exit(&md5, buffer, sizeof(buffer));
 
             // make md5 string
-            tb_size_t i = 0;
-            tb_char_t s[256] = {0};
-            for (i = 0; i < 16; ++i) tb_snprintf(s + (i << 1), 3, "%02x", buffer[i]);
+            tb_char_t s[256];
+            tb_size_t n = xm_hash_make_cstr(s, buffer, 16);
 
             // save result
-	        lua_pushstring(lua, s);
-
-            // ok
+	        lua_pushlstring(lua, s, n);
             ok = tb_true;
         }
 
