@@ -12,7 +12,7 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  *
- * Copyright (C) 2015-present, TBOOX Open Source Group.
+ * Copyright (C) 2015-present, Xmake Open Source Community.
  *
  * @author      ruki
  * @file        malloc.c
@@ -42,7 +42,7 @@ tb_int_t xm_libc_malloc(lua_State* lua)
     tb_pointer_t data = tb_null;
     tb_long_t size = (tb_long_t)luaL_checkinteger(lua, 1);
     if (size > 0) data = tb_malloc(size);
-    lua_pushinteger(lua, (lua_Integer)data);
+    lua_pushinteger(lua, (lua_Integer)(tb_long_t)data);
     return 1;
 }
 
