@@ -12,7 +12,7 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  *
- * Copyright (C) 2015-present, TBOOX Open Source Group.
+ * Copyright (C) 2015-present, Xmake Open Source Community.
  *
  * @author      ruki
  * @file        cpfile.c
@@ -53,6 +53,10 @@ tb_int_t xm_os_cpfile(lua_State* lua)
     if (is_writeable)
         flags |= TB_FILE_COPY_WRITEABLE;
 
+    tb_bool_t copy_if_different = lua_toboolean(lua, 5);
+    if (copy_if_different)
+        flags |= TB_FILE_COPY_IF_DIFFERENT;
+
     // do copy
     lua_pushboolean(lua, tb_file_copy(src, dst, flags));
     return 1;
