@@ -12,7 +12,7 @@
 -- See the License for the specific language governing permissions and
 -- limitations under the License.
 --
--- Copyright (C) 2015-present, TBOOX Open Source Group.
+-- Copyright (C) 2015-present, Xmake Open Source Community.
 --
 -- @author      ruki
 -- @file        os.lua
@@ -39,6 +39,7 @@ os._mkdir    = os._mkdir or os.mkdir
 os._rmdir    = os._rmdir or os.rmdir
 os._touch    = os._touch or os.touch
 os._tmpdir   = os._tmpdir or os.tmpdir
+os._curdir   = os._curdir or os.curdir
 os._fscase   = os._fscase or os.fscase
 os._setenv   = os._setenv or os.setenv
 os._getenvs  = os._getenvs or os.getenvs
@@ -71,6 +72,7 @@ function os._cp(src, dst, rootdir, opt)
     -- is file or link?
     local symlink = opt.symlink
     local writeable = opt.writeable
+    local copy_if_different = opt.copy_if_different
     if os.isfile(src) or (symlink and os.islink(src)) then
 
         -- the destination is directory? append the filename
@@ -86,7 +88,7 @@ function os._cp(src, dst, rootdir, opt)
         if opt.force and os.isfile(dst) then
             os.rmfile(dst)
         end
-        if not os.cpfile(src, dst, symlink, writeable) then
+        if not os.cpfile(src, dst, symlink, writeable, copy_if_different) then
             local errors = os.strerror()
             if symlink and os.islink(src) then
                 local reallink = os.readlink(src)
@@ -108,7 +110,7 @@ function os._cp(src, dst, rootdir, opt)
         end
 
         -- copy directory
-        if not os.cpdir(src, dst, symlink) then
+        if not os.cpdir(src, dst, symlink, copy_if_different) then
             return false, string.format("cannot copy directory %s to %s,  %s", src, dst, os.strerror())
         end
     else
@@ -197,6 +199,14 @@ function os._sched_chdir_set(chdir)
     os._SCHED_CHDIR = chdir
 end
 
+-- notify the current directory have been changed
+function os._notify_curdir_changed()
+    os._CURDIR = nil
+    if os._SCHED_CHDIR then
+        os._SCHED_CHDIR(os.curdir())
+    end
+end
+
 -- notify envs have been changed
 function os._notify_envs_changed(envs)
     if os._SCHED_CHENVS then
@@ -488,6 +498,11 @@ function os.filedirs(pattern, callback)
 end
 
 -- copy files or directories and we can reserve the source directory structure
+--
+-- @param srcpath   the source file path
+-- @param dstpath   the destination file path
+-- @param opt       the copy option. e.g. {rootdir, symlink, writeable, force, copy_if_different}
+--
 -- e.g. os.cp("src/**.h", "/tmp/", {rootdir = "src", symlink = true})
 function os.cp(srcpath, dstpath, opt)
 
@@ -605,41 +620,33 @@ function os.cd(dir)
     -- support path instance
     dir = tostring(dir)
 
-    -- the previous directory
-    local oldir = os.curdir()
-
     -- change to the previous directory?
+    local oldir = os.curdir()
     if dir == "-" then
-        -- exists the previous directory?
         if os._PREDIR then
             dir = os._PREDIR
             os._PREDIR = nil
         else
-            -- error
             return nil, string.format("not found the previous directory %s", os.strerror())
         end
     end
 
-    -- is directory?
-    if os.isdir(dir) then
+    -- no changed?
+    if dir == oldir then
+        return oldir
+    end
 
-        -- change to directory
+    -- do change directory
+    if os.isdir(dir) then
         if not os.chdir(dir) then
             return nil, string.format("cannot change directory %s %s", dir, os.strerror())
         end
-
-        -- save the previous directory
         os._PREDIR = oldir
-
-    -- not exists?
     else
         return nil, string.format("cannot change directory %s, not found this directory %s", dir, os.strerror())
     end
 
-    -- do chdir callback for scheduler
-    if os._SCHED_CHDIR then
-        os._SCHED_CHDIR(os.curdir())
-    end
+    os._notify_curdir_changed()
     return oldir
 end
 
@@ -695,6 +702,16 @@ function os.rmdir(dir)
     return true
 end
 
+-- get the current directory
+function os.curdir()
+    local curdir = os._CURDIR
+    if curdir == nil then
+        curdir = os._curdir()
+        os._CURDIR = curdir
+    end
+    return curdir
+end
+
 -- get the temporary directory
 function os.tmpdir(opt)
 
@@ -751,7 +768,8 @@ function os.tmpfile(opt_or_key)
         key = opt_or_key.key
         opt = opt_or_key
     end
-    return path.join(os.tmpdir(opt), "_" .. (hash.uuid4(key):gsub("-", "")))
+    local filename = "_" .. (key and hash.strhash128(key) or (hash.rand128()))
+    return path.join(os.tmpdir(opt), filename)
 end
 
 -- exit program
@@ -1289,23 +1307,29 @@ function os.term()
     return require("base/tty").term()
 end
 
--- get all current environment variables
--- e.g. envs["PATH"] = "/xxx:/yyy/foo"
+-- get all current environments variables
 function os.getenvs()
-    local envs = {}
-    for _, line in ipairs(os._getenvs()) do
-        local p = line:find('=', 1, true)
-        if p then
-            local key = line:sub(1, p - 1):trim()
-            -- only translate Path to PATH on windows
-            -- @see https://github.com/xmake-io/xmake/issues/3752
-            if os.host() == "windows" and key:lower() == "path" then
-                key = key:upper()
-            end
-            local values = line:sub(p + 1):trim()
-            if #key > 0 then
-                envs[key] = values
+    local envs = os._getenvs()
+    if envs then
+        -- we need to be compatible with the old binary core, if it's array (<= v3.0.3)
+        if envs[1] ~= nil then
+            local result = {}
+            for _, line in ipairs(envs) do
+                local p = line:find('=', 1, true)
+                if p then
+                    local key = line:sub(1, p - 1):trim()
+                    -- only translate Path to PATH on windows
+                    -- @see https://github.com/xmake-io/xmake/issues/3752
+                    if os.host() == "windows" and key:lower() == "path" then
+                        key = key:upper()
+                    end
+                    local values = line:sub(p + 1):trim()
+                    if #key > 0 then
+                        result[key] = values
+                    end
+                end
             end
+            envs = result
         end
     end
     return envs
