@@ -12,7 +12,7 @@
 -- See the License for the specific language governing permissions and
 -- limitations under the License.
 --
--- Copyright (C) 2015-present, TBOOX Open Source Group.
+-- Copyright (C) 2015-present, Xmake Open Source Community.
 --
 -- @author      ruki
 -- @file        xmake.lua
@@ -62,6 +62,11 @@ function xmake.luajit()
     return xmake._LUAJIT
 end
 
+-- is embed?
+function xmake.is_embed()
+    return xmake._EMBED or false
+end
+
 -- get command arguments
 function xmake.argv()
     return xmake._ARGV
