@@ -12,7 +12,7 @@
 -- See the License for the specific language governing permissions and
 -- limitations under the License.
 --
--- Copyright (C) 2015-present, TBOOX Open Source Group.
+-- Copyright (C) 2015-present, Xmake Open Source Community.
 --
 -- @author      ruki
 -- @file        project.lua
@@ -25,6 +25,7 @@ local project = project or {}
 local os                    = require("base/os")
 local io                    = require("base/io")
 local path                  = require("base/path")
+local hash                  = require("base/hash")
 local task                  = require("base/task")
 local utils                 = require("base/utils")
 local table                 = require("base/table")
@@ -417,7 +418,8 @@ function project._load_targets()
         for _, sourcefile in ipairs(table.wrap(t:get("files"))) do
             local extension = path.extension((sourcefile:gsub("|.*$", "")))
             if not extensions[extension] then
-                local lang = language.load_ex(extension)
+                local sourcekind = t:extraconf("files", sourcefile, "sourcekind")
+                local lang = sourcekind and language.load_sk(sourcekind) or language.load_ex(extension)
                 if lang and lang:rules() then
                     table.join2(rulenames, lang:rules())
                 end
@@ -734,9 +736,12 @@ function project.interpreter()
         -- check
         assert(variable)
 
-        -- hack buildir first
-        if variable == "buildir" then
-            return config.buildir()
+        -- hack builddir first
+        if variable == "builddir" or variable == "buildir" then
+            if variable == "buildir" then
+                utils.warning("$(buildir) has been deprecated, please use $(builddir)")
+            end
+            return config.builddir()
         end
 
         -- attempt to get it directly from the configure
@@ -885,6 +890,7 @@ function project._init_default_policies()
         if semver.compare(compatibility_version, "3.0") >= 0 then
             policy.set_default("package.cmake_generator.ninja", true)
             policy.set_default("build.c++.msvc.runtime", "MD")
+            policy.set_default("run.autobuild", true)
         else
             policy.set_default("package.cmake_generator.ninja", false)
             policy.set_default("build.c++.msvc.runtime", "MT")
@@ -1088,7 +1094,7 @@ function project.requires_str()
                 local ns_requires_str, ns_requires_extra = project.get(namespace .. "::requires"), project.get(namespace .. "::__extra_requires")
                 if ns_requires_str then
                     requires_str = table.wrap(requires_str)
-                    table.insert(requires_str, ns_requires_str)
+                    table.join2(requires_str, ns_requires_str)
                 end
                 if ns_requires_extra then
                     requires_extra = table.wrap(requires_extra)
@@ -1106,7 +1112,7 @@ function project.requires_str()
                 local ns_requireconfs_str, ns_requireconfs_extra = project.get(namespace .. "::requireconfs"), project.get(namespace .. "::__extra_requireconfs")
                 if ns_requireconfs_str then
                     requireconfs_str = table.wrap(requireconfs_str)
-                    table.insert(requireconfs_str, ns_requireconfs_str)
+                    table.join2(requireconfs_str, ns_requireconfs_str)
                 end
                 if ns_requireconfs_extra then
                     requireconfs_extra = table.wrap(requireconfs_extra)
@@ -1375,7 +1381,8 @@ function project.tmpfile(opt_or_key)
         key = opt_or_key.key
         opt = opt_or_key
     end
-    return path.join(project.tmpdir(opt), "_" .. (hash.uuid4(key):gsub("-", "")))
+    local filename = "_" .. (key and hash.strhash128(key) or (hash.rand128()))
+    return path.join(project.tmpdir(opt), "_" .. filename)
 end
 
 -- get all modes
