@@ -12,7 +12,7 @@
 -- See the License for the specific language governing permissions and
 -- limitations under the License.
 --
--- Copyright (C) 2015-present, TBOOX Open Source Group.
+-- Copyright (C) 2015-present, Xmake Open Source Community.
 --
 -- @author      ruki
 -- @file        project.lua
@@ -154,6 +154,15 @@ function sandbox_core_project._config_target(target, opt)
     end
 end
 
+function sandbox_core_project._config_targets(opt)
+    opt = opt or {}
+    for _, target in ipairs(table.wrap(project.ordertargets())) do
+        if target:is_enabled() then
+            sandbox_core_project._config_target(target, opt)
+        end
+    end
+end
+
 -- config targets
 --
 -- @param opt   the extra option, e.g. {recheck = false}
@@ -164,31 +173,39 @@ end
 --        target:has_cfuncs(...)
 --    end
 -- end
---
+-- TODO we need to optimize jobgraph
+-- https://github.com/xmake-io/xmake/issues/6775
+--[[
 function sandbox_core_project._config_targets(opt)
-    opt = opt or {}
-    for _, target in ipairs(table.wrap(project.ordertargets())) do
-        if target:is_enabled() then
-            sandbox_core_project._config_target(target, opt)
-        end
-    end
-end
+    import("private.action.build.target", {alias = "target_buildutils"})
+
+    -- we need to config all targets (contains non-default targets)
+    --
+    -- @note Currently, many configurations still depend on the order of rules/add_deps (e.g. qt rules),
+    -- so we can only execute single tasks for now (jobs = 1).
+    local targets_root = target_buildutils.get_root_targets(nil, {all = true})
+    target_buildutils.run_targetjobs(targets_root, {job_kind = "config", target_fence = true, jobs = 1, job_opt = opt})
+end]]
 
 -- load rules in the required packages for target
 function sandbox_core_project._load_package_rules_for_target(target)
     for _, rulename in ipairs(table.wrap(target:get("rules"))) do
         local packagename = rulename:match("@(.-)/")
         if packagename then
+            local ruleinst
             local pkginfo = project.required_package(packagename)
             if pkginfo then
-                local r = pkginfo:rule(rulename)
-                if r then
-                    target:rule_add(r)
-                    for _, dep in pairs(table.wrap(r:deps())) do
+                ruleinst = pkginfo:rule(rulename)
+                if ruleinst then
+                    target:rule_add(ruleinst)
+                    for _, dep in pairs(table.wrap(ruleinst:deps())) do
                         target:rule_add(dep)
                     end
                 end
             end
+            if not ruleinst then
+                raise("target(\"%s\"): unknown package rule in add_rules(\"%s\")", target:fullname(), rulename)
+            end
         end
     end
 end
