@@ -12,7 +12,7 @@
 -- See the License for the specific language governing permissions and
 -- limitations under the License.
 --
--- Copyright (C) 2015-present, TBOOX Open Source Group.
+-- Copyright (C) 2015-present, Xmake Open Source Community.
 --
 -- @author      ruki
 -- @file        cl.lua
@@ -26,7 +26,7 @@ import("core.project.project")
 import("core.project.policy")
 import("core.language.language")
 import("private.tools.vstool")
-import("private.tools.cl.parse_include")
+import("core.tools.cl.parse_include")
 import("private.cache.build_cache")
 import("private.service.distcc_build.client", {alias = "distcc_build_client"})
 import("utils.progress")
@@ -76,10 +76,16 @@ function init(self)
 
         -- language
     ,   ["-ansi"]                   = ""
-    ,   ["-std=c99"]                = "-TP" -- compile as c++ files because msvc only support c89
-    ,   ["-std=c11"]                = "-TP" -- compile as c++ files because msvc only support c89
-    ,   ["-std=gnu99"]              = "-TP" -- compile as c++ files because msvc only support c89
-    ,   ["-std=gnu11"]              = "-TP" -- compile as c++ files because msvc only support c89
+    ,   ["-std=c99"]                = "-TP" -- compile as c++ files because msvc doesn't support c99
+    ,   ["-std=c11"]                = "-std:c11"
+    ,   ["-std=c17"]                = "-std:c17"
+    ,   ["-std=c2x"]                = "-std:clatest"
+    ,   ["-std=c23"]                = "-std:clatest"
+    ,   ["-std=gnu99"]              = "-TP" -- compile as c++ files because msvc doesn't support c99
+    ,   ["-std=gnu11"]              = "-std:c11"
+    ,   ["-std=gnu17"]              = "-std:c17"
+    ,   ["-std=gnu2x"]              = "-std:clatest"
+    ,   ["-std=gnu23"]              = "-std:clatest"
     ,   ["-std=.*"]                 = ""
 
         -- others
@@ -177,9 +183,9 @@ function nf_optimize(self, level)
     {
         none        = "-Od"
     ,   faster      = "-Ox"
-    ,   fastest     = "-O2 -fp:fast"
+    ,   fastest     = "-O2"
     ,   smallest    = "-O1 -GL" -- /GL and (/OPT:REF is on by default in linker), we need to enable /ltcg
-    ,   aggressive  = "-O2 -fp:fast"
+    ,   aggressive  = "-O2"
     }
     return maps[level]
 end
@@ -210,9 +216,22 @@ function nf_vectorext(self, extension)
     ,   fma        = "-arch:AVX2"
     ,   all        = {"-arch:SSE", "-arch:SSE2", "/d2archSSE42", "-arch:AVX", "-arch:AVX2", "-arch:AVX512"}
     }
-    local flag = maps[extension]
-    if flag and self:has_flags(flag, "cxflags") then
-        return flag
+    local flags = maps[extension]
+    if flags then
+        -- @see https://github.com/xmake-io/xmake/issues/5499
+        if type(flags) == "string" then
+            return flags
+        else
+            local result = {}
+            for _, flag in ipairs(flags) do
+                if self:has_flags(flag, "cxflags") then
+                    table.insert(result, flag)
+                end
+            end
+            if #result > 0 then
+                return table.unwrap(result)
+            end
+        end
     end
 end
 
@@ -227,12 +246,16 @@ function nf_language(self, stdname)
             -- stdc
             c99       = "-TP" -- compile as c++ files because older msvc only support c89
         ,   gnu99     = "-TP"
-        ,   c11       = {"-std:c11", "-TP"}
-        ,   gnu11     = {"-std:c11", "-TP"}
-        ,   c17       = {"-std:c17", "-TP"}
-        ,   gnu17     = {"-std:c17", "-TP"}
-        ,   clatest   = {"-std:c17", "-std:c11"}
-        ,   gnulatest = {"-std:c17", "-std:c11"}
+        ,   c11       = {"-std:c11", "-std:clatest", "-TP"}
+        ,   gnu11     = {"-std:c11", "-std:clatest", "-TP"}
+        ,   c17       = {"-std:c17", "-std:clatest", "-TP"}
+        ,   gnu17     = {"-std:c17", "-std:clatest", "-TP"}
+        ,   c2x       = {"-std:c23", "-std:clatest", "-TP"}
+        ,   gnu2x     = {"-std:c23", "-std:clatest", "-TP"}
+        ,   c23       = {"-std:c23", "-std:clatest", "-TP"}
+        ,   gnu23     = {"-std:c23", "-std:clatest", "-TP"}
+        ,   clatest   = {"-std:clatest", "-std:c23", "-std:c17", "-std:c11", "-TP"}
+        ,   gnulatest   = {"-std:clatest", "-std:c23", "-std:c17", "-std:c11", "-TP"}
         }
     end
 
@@ -256,6 +279,8 @@ function nf_language(self, stdname)
         ,   gnuxx23     = {"-std:c++23", "-std:c++latest"}
         ,   cxx2b       = {"-std:c++23", "-std:c++latest"}
         ,   gnuxx2b     = {"-std:c++23", "-std:c++latest"}
+        ,   cxx26       = {"-std:c++26", "-std:c++latest"}
+        ,   gnuxx26     = {"-std:c++26", "-std:c++latest"}
         ,   cxxlatest   = "-std:c++latest"
         ,   gnuxxlatest = "-std:c++latest"
         }
@@ -363,7 +388,8 @@ function nf_encoding(self, encoding)
     end
     local charsets = {
         ["utf-8"] = "utf-8",
-        utf8 = "utf-8"
+        utf8 = "utf-8",
+        gb2312 = "gb2312"
     }
     local flags = {}
     charset = charsets[charset:lower()]
@@ -421,7 +447,7 @@ function add_sourceflags(self, sourcefile, fileconfig, target, targetkind)
     --   add_files("*.c", {sourcekind = "cxx"})
     --
     local sourcekind = fileconfig.sourcekind
-    if sourcekind and sourcekind ~= language.sourcekind_of(sourcefile) then
+    if sourcekind then
         local maps = {cc = "-TC", cxx = "-TP"}
         return maps[sourcekind]
     end
@@ -752,10 +778,13 @@ function compile(self, sourcefile, objectfile, dependinfo, flags, opt)
     -- generate the dependent includes
     if dependinfo then
         if depfile and os.isfile(depfile) then
-            dependinfo.depfiles_cl_json = io.readfile(depfile)
+            dependinfo.depfiles_format = "cl_json"
+            dependinfo.depfiles = io.readfile(depfile)
             os.tryrm(depfile)
         elseif outdata then
-            dependinfo.depfiles_cl = outdata
+            dependinfo.depfiles_format = "cl"
+            dependinfo.depfiles = outdata
         end
     end
 end
+
