@@ -21,6 +21,7 @@
 -- imports
 import("lib.detect.find_program")
 import("lib.detect.find_programver")
+import("detect.tools.find_gcc")
 
 -- find c++
 --
@@ -35,15 +36,10 @@ import("lib.detect.find_programver")
 -- @endcode
 --
 function main(opt)
-
-    -- init options
     opt = opt or {}
 
-    -- find program
-    local program = find_program(opt.program or "cc", opt)
-
-    -- find program version
     local version = nil
+    local program = find_program(opt.program or "cc", opt)
     if program and opt and opt.version then
         version = find_programver(program, opt)
     end
@@ -51,10 +47,7 @@ function main(opt)
     -- is clang++ or c++
     local is_clang = false
     if program then
-        local versioninfo = os.iorunv(program, {"--version"})
-        if versioninfo and versioninfo:find("clang", 1, true) then
-            is_clang = true
-        end
+        is_clang = find_gcc.check_clang(program, opt)
     end
     return program, version, (is_clang and "clang" or "cc")
 end
