@@ -12,7 +12,7 @@
 -- See the License for the specific language governing permissions and
 -- limitations under the License.
 --
--- Copyright (C) 2015-present, TBOOX Open Source Group.
+-- Copyright (C) 2015-present, Xmake Open Source Community.
 --
 -- @author      ruki
 -- @file        ninja.lua
@@ -24,16 +24,19 @@ import("lib.detect.find_tool")
 
 function _default_argv(package, configs, opt)
     opt = opt or {}
-    local buildir = opt.buildir or os.curdir()
+    local builddir = opt.builddir or opt.buildir or os.curdir()
     local njob = opt.jobs or option.get("jobs") or tostring(os.default_njob())
+    if opt.buildir then
+        wprint("{buildir = } has been deprecated, please use {builddir = } in ninja.install")
+    end
 
     local argv = {}
-    local targets = table.wrap(opt.target)
+    local targets = table.wrap(opt.targets or opt.target)
     if #targets ~= 0 then
         table.join2(argv, targets)
     end
     table.insert(argv, "-C")
-    table.insert(argv, buildir)
+    table.insert(argv, builddir)
     if option.get("diagnosis") then
         table.insert(argv, "-v")
     end
