@@ -12,7 +12,7 @@
 -- See the License for the specific language governing permissions and
 -- limitations under the License.
 --
--- Copyright (C) 2015-present, TBOOX Open Source Group.
+-- Copyright (C) 2015-present, Xmake Open Source Community.
 --
 -- @author      ruki
 -- @file        xmake.lua
@@ -270,30 +270,24 @@ function _get_configs(package, configs, opt)
     end
 
     local policies = get_config("policies")
+    local policies_list = policies and policies:split(",") or {}
     if package:config("lto") and (not policies or not policies:find("build.optimization.lto", 1, true)) then
-        if policies then
-            policies = policies .. ",build.optimization.lto"
-        else
-            policies = "build.optimization.lto"
-        end
+        table.insert(policies_list, "build.optimization.lto")
     end
     if package:config("asan") and (not policies or not policies:find("build.sanitizer.address", 1, true)) then
-        if policies then
-            policies = policies .. ",build.sanitizer.address"
-        else
-            policies = "build.sanitizer.address"
-        end
+        table.insert(policies_list, "build.sanitizer.address")
     end
     if not package:use_external_includes() and (not policies or not policies:find("package.include_external_headers", 1, true)) then
-        if policies then
-            policies = policies .. ",package.include_external_headers:n"
-        else
-            policies = "package.include_external_headers:n"
-        end
+        table.insert(policies_list, "package.include_external_headers:n")
     end
-    if policies then
-        table.insert(configs, "--policies=" .. policies)
+    if policies and policies:find("package.build.ccache", 1, true) then
+        table.insert(configs, "--ccachedir=" .. path.join(path.directory(package:cachedir()), "build_cache"))
+        table.insert(policies_list, "build.ccache")
     end
+    if #policies_list > 0 then
+        table.insert(configs, "--policies=" .. table.concat(policies_list, ","))
+    end
+
     if not package:is_plat("windows", "mingw") and package:config("pic") ~= false then
         table.insert(cxflags, "-fPIC")
     end
@@ -315,9 +309,12 @@ function _get_configs(package, configs, opt)
     if shflags and #shflags > 0 then
         table.insert(configs, "--shflags=" .. table.concat(shflags, ' '))
     end
-    local buildir = opt.buildir or package:buildir()
-    if buildir then
-        table.insert(configs, "--buildir=" .. buildir)
+    local builddir = opt.builddir or opt.buildir or package:builddir()
+    if builddir then
+        table.insert(configs, "--builddir=" .. builddir)
+    end
+    if opt.buildir then
+        wprint("{buildir = } has been deprecated, please use {builddir = } in xmake.install")
     end
     return configs
 end
@@ -484,9 +481,18 @@ end
 
 -- install package
 function install(package, configs, opt)
+    opt = opt or {}
+
+    -- copy the ported xmake.lua in the default position if it's missing
+    local xmakefile = path.join(opt.curdir or os.curdir(), "xmake.lua")
+    if not os.isfile(xmakefile) and package:repo() ~= nil then
+        local xmakefile_port = path.join(package:scriptdir(), "port", "xmake.lua")
+        if os.isfile(xmakefile_port) then
+            os.cp(xmakefile_port, xmakefile)
+        end
+    end
 
     -- get build environments
-    opt = opt or {}
     local envs = opt.envs or buildenvs(package)
 
     -- pass local repositories
@@ -494,7 +500,7 @@ function install(package, configs, opt)
         local repo_argv = {"repo"}
         _set_builtin_argv(package, repo_argv)
         table.join2(repo_argv, {"--add", repo:name(), repo:directory()})
-        os.vrunv(os.programfile(), repo_argv, {envs = envs})
+        os.vrunv(os.programfile(), repo_argv, {envs = envs, curdir = opt.curdir})
     end
 
     -- pass configurations
@@ -515,7 +521,7 @@ function install(package, configs, opt)
     end
 
     -- do configure
-    os.vrunv(os.programfile(), argv, {envs = envs})
+    os.vrunv(os.programfile(), argv, {envs = envs, curdir = opt.curdir})
 
     -- do build
     argv = {"build"}
@@ -524,18 +530,18 @@ function install(package, configs, opt)
     if njob then
         table.insert(argv, "--jobs=" .. njob)
     end
-    local target = table.wrap(opt.target)
-    if #target ~= 0 then
-        table.join2(argv, target)
-    end
-    os.vrunv(os.programfile(), argv, {envs = envs})
-
-    -- do install
-    argv = {"install", "-y", "--nopkgs", "-o", package:installdir()}
-    _set_builtin_argv(package, argv)
-    local targets = table.wrap(opt.target)
+    local targets = table.wrap(opt.targets or opt.target)
     if #targets ~= 0 then
         table.join2(argv, targets)
     end
-    os.vrunv(os.programfile(), argv, {envs = envs})
+    os.vrunv(os.programfile(), argv, {envs = envs, curdir = opt.curdir})
+
+    -- do install
+    argv = {"install", "-y", "--packages=n", "-o", package:installdir()}
+    _set_builtin_argv(package, argv)
+    local targets = table.wrap(opt.targets or opt.target)
+    if #targets ~= 0 then
+        table.join2(argv, targets)
+    end
+    os.vrunv(os.programfile(), argv, {envs = envs, curdir = opt.curdir})
 end
