@@ -12,7 +12,7 @@
 -- See the License for the specific language governing permissions and
 -- limitations under the License.
 --
--- Copyright (C) 2015-present, TBOOX Open Source Group.
+-- Copyright (C) 2015-present, Xmake Open Source Community.
 --
 -- @author      ruki
 -- @file        download.lua
@@ -27,6 +27,7 @@ import("core.project.config")
 import("core.package.package", {alias = "core_package"})
 import("lib.detect.find_file")
 import("lib.detect.find_directory")
+import("private.action.require.impl.check_api")
 import("private.action.require.impl.utils.filter")
 import("private.action.require.impl.utils.url_filename")
 import("net.http")
@@ -34,9 +35,21 @@ import("net.proxy")
 import("devel.git")
 import("utils.archive")
 
+-- get url extension
+function _url_extension(url)
+    local extension = archive.extension(url)
+    if extension == "" then
+        -- maybe non-archive file, e.g. .exe, .sh, ..
+        local urlpath = url:split('?', {plain = true})[1]
+        extension = path.extension(urlpath)
+    end
+    return extension
+end
+
 -- checkout codes from git
 function _checkout(package, url, sourcedir, opt)
     opt = opt or {}
+    local filename = opt.url_filename or url_filename(url)
 
     -- we need to enable longpaths on windows
     local longpaths = package:policy("platform.longpaths")
@@ -62,7 +75,7 @@ function _checkout(package, url, sourcedir, opt)
     -- we can use local package from the search directories directly if network is too slow
     local localdir
     local searchnames = {package:name() .. archive.extension(url),
-                         path.basename(url_filename(url))}
+                         path.basename(filename)}
     for _, searchname in ipairs(searchnames) do
         localdir = find_directory(searchname, core_package.searchdirs())
         if localdir then
@@ -95,8 +108,9 @@ function _checkout(package, url, sourcedir, opt)
             branch = nil
         end
 
-        -- only shallow clone this branch
-        git.clone(url, {depth = 1, recursive = true, shallow_submodules = true, longpaths = longpaths, branch = branch, outputdir = packagedir})
+        -- only shallow clone this branch 
+        local clone_submodules = opt.url_submodules ~= false
+        git.clone(url, {depth = 1, recursive = clone_submodules, shallow_submodules = clone_submodules, longpaths = longpaths, branch = branch, outputdir = packagedir})
 
     -- download package from revision or tag?
     else
@@ -117,14 +131,16 @@ function _checkout(package, url, sourcedir, opt)
         -- only shallow clone this tag
         -- @see https://github.com/xmake-io/xmake/issues/4151
         if tag and git.clone.can_clone_tag() then
-            git.clone(url, {depth = 1, recursive = true, shallow_submodules = true, longpaths = longpaths, branch = tag, outputdir = packagedir})
+            local clone_submodules = opt.url_submodules ~= false
+            git.clone(url, {depth = 1, recursive = clone_submodules, shallow_submodules = clone_submodules, longpaths = longpaths, branch = tag, outputdir = packagedir})
         else
 
             -- clone whole history and tags
-            git.clone(url, {longpaths = longpaths, outputdir = packagedir})
+            -- @see https://github.com/xmake-io/xmake/issues/5507
+            git.clone(url, {treeless = true, checkout = false, longpaths = longpaths, outputdir = packagedir})
 
             -- attempt to checkout the given version
-            git.checkout(revision, {repodir = packagedir})
+            git.checkout(revision, {repodir = packagedir, includes = opt.url_includes})
 
             -- update all submodules
             if os.isfile(path.join(packagedir, ".gitmodules")) and opt.url_submodules ~= false then
@@ -147,9 +163,12 @@ function _download(package, url, sourcedir, opt)
     opt = opt or {}
 
     -- get package file
-    local packagefile = url_filename(url)
-    if opt.download_only then
-        packagefile = package:name() .. "-" .. package:version_str() .. archive.extension(packagefile)
+    local packagefile = opt.url_filename
+    if not packagefile then
+        packagefile = url_filename(url)
+        if not os.isfile(packagefile) then -- we need to be compatible with the old file names
+            packagefile = package:name() .. "-" .. package:version_str() .. _url_extension(packagefile)
+        end
     end
 
     -- get sourcehash from the given url
@@ -176,7 +195,7 @@ function _download(package, url, sourcedir, opt)
             os.cp(url, packagefile)
         else
             local localfile
-            local searchnames = {package:name() .. "-" .. package:version_str() .. archive.extension(url),
+            local searchnames = {package:name() .. "-" .. package:version_str() .. _url_extension(url),
                                  packagefile}
 
             -- match github name mangling https://github.com/xmake-io/xmake/issues/1343
@@ -212,6 +231,12 @@ function _download(package, url, sourcedir, opt)
 
     -- we do not extract it if we download only it.
     if opt.download_only then
+        tty.erase_line_to_start().cr()
+        if cached then
+            cprint("${yellow}  => ${clear}download %s .. ${color.success}cached", url)
+        else
+            cprint("${yellow}  => ${clear}download %s .. ${color.success}${text.success}", url)
+        end
         return
     end
 
@@ -219,7 +244,20 @@ function _download(package, url, sourcedir, opt)
     local sourcedir_tmp = sourcedir .. ".tmp"
     os.rm(sourcedir_tmp)
     local extension = archive.extension(packagefile)
-    local ok = try {function() archive.extract(packagefile, sourcedir_tmp, {excludes = opt.url_excludes}); return true end}
+    local errors
+    local ok = try {
+        function()
+            archive.extract(packagefile, sourcedir_tmp, {excludes = opt.url_excludes})
+            return true
+        end,
+        catch {
+            function (errs)
+                if errs then
+                    errors = tostring(errs)
+                end
+            end
+        }
+    }
     if ok then
         -- move to source directory and we skip it to avoid long path issues on windows if only one root directory
         os.rm(sourcedir)
@@ -240,9 +278,10 @@ function _download(package, url, sourcedir, opt)
         -- create an empty source directory if do not extract package file
         os.tryrm(sourcedir)
         os.mkdir(sourcedir)
-        raise("cannot extract %s, maybe missing extractor or invalid package file!", packagefile)
+        raise(errors or string.format("cannot extract %s, maybe missing extractor or invalid package file!", packagefile))
     else
         -- if it is not archive file, we only need to create empty source directory and use package:originfile()
+        -- e.g. .exe, .sh
         os.tryrm(sourcedir)
         os.mkdir(sourcedir)
     end
@@ -259,11 +298,7 @@ end
 
 -- download codes from script
 function _download_from_script(package, script, opt)
-
-    -- do download
     script(package, opt)
-
-    -- trace
     tty.erase_line_to_start().cr()
     cprint("${yellow}  => ${clear}download %s .. ${color.success}${text.success}", opt.url)
 end
@@ -313,8 +348,10 @@ function main(package, opt)
     for idx, url in ipairs(urls) do
         local url_alias = package:url_alias(url)
         local url_excludes = package:url_excludes(url)
+        local url_includes = package:url_includes(url)
         local url_http_headers = package:url_http_headers(url)
         local url_submodules = package:extraconf("urls", url, "submodules")
+        local url_filename_custom = package:extraconf("urls", url, "filename")
 
         -- filter url
         url = filter.handle(url, package)
@@ -348,17 +385,23 @@ function main(package, opt)
                         url = url,
                         url_alias = url_alias,
                         url_excludes = url_excludes,
-                        url_submodules = url_submodules})
+                        url_includes = url_includes,
+                        url_submodules = url_submodules,
+                        url_filename = url_filename_custom})
                 elseif git.checkurl(url) then
                     _checkout(package, url, sourcedir, {
                         url_alias = url_alias,
-                        url_submodules = url_submodules})
+                        url_includes = url_includes,
+                        url_submodules = url_submodules,
+                        url_filename = url_filename_custom})
                 else
                     _download(package, url, sourcedir, {
                         download_only = opt.download_only,
                         url_alias = url_alias,
                         url_excludes = url_excludes,
-                    url_http_headers = url_http_headers})
+                        url_includes = url_includes,
+                        url_http_headers = url_http_headers,
+                        url_filename = url_filename_custom})
                 end
                 return true
             end,
@@ -366,6 +409,8 @@ function main(package, opt)
             {
                 function (errors)
 
+                    check_api(package, {download_failure = true})
+
                     -- show or save the last errors
                     if errors then
                         if (option.get("verbose") or option.get("diagnosis")) then
@@ -396,7 +441,7 @@ function main(package, opt)
                                     searchnames:insert(package:name() .. archive.extension(url_failed))
                                     searchnames:insert(path.basename(url_filename(url_failed)))
                                 else
-                                    local extension = archive.extension(url_failed)
+                                    local extension = _url_extension(url_failed)
                                     if extension then
                                         searchnames:insert(package:name() .. "-" .. package:version_str() .. extension)
                                     end
@@ -418,15 +463,6 @@ function main(package, opt)
         }
 
         if ok then
-            -- download only packages, we need not to install it, so we need flush console output
-            if opt.download_only then
-                tty.erase_line_to_start().cr()
-                if git.checkurl(url) then
-                    cprint("${yellow}  => ${clear}clone %s %s .. ${color.success}${text.success}", url, package:version_str())
-                else
-                    cprint("${yellow}  => ${clear}download %s .. ${color.success}${text.success}", url)
-                end
-            end
             break
         end
     end
@@ -438,5 +474,3 @@ function main(package, opt)
     os.cd(oldir)
     return ok
 end
-
-
