@@ -12,7 +12,7 @@
 -- See the License for the specific language governing permissions and
 -- limitations under the License.
 --
--- Copyright (C) 2015-present, TBOOX Open Source Group.
+-- Copyright (C) 2015-present, Xmake Open Source Community.
 --
 -- @author      ruki
 -- @file        install_packages.lua
@@ -167,7 +167,8 @@ function _get_confirm_from_3rd(packages)
 end
 
 -- get user confirm
-function _get_confirm(packages)
+function _get_confirm(packages, opt)
+    opt = opt or {}
 
     -- no confirmed packages?
     if #packages == 0 then
@@ -201,7 +202,11 @@ function _get_confirm(packages)
             end
 
             -- show tips
-            cprint("${bright color.warning}note: ${clear}install or modify (m) these packages (pass -y to skip confirm)?")
+            if opt.toolchain then
+                cprint("${bright color.warning}note: ${clear}install or modify (m) these ${bright}toolchain${clear} packages first (pass -y to skip confirm)?")
+            else
+                cprint("${bright color.warning}note: ${clear}install or modify (m) these packages (pass -y to skip confirm)?")
+            end
             for reponame, packages in pairs(packages_repo) do
                 if reponame ~= "" then
                     print("in %s:", reponame)
@@ -212,12 +217,26 @@ function _get_confirm(packages)
                         local group = instance:group()
                         if group and packages_group[group] and #packages_group[group] > 1 then
                             for idx, package_in_group in ipairs(packages_group[group]) do
-                                cprint("  ${yellow}%s${clear} %s %s ${dim}%s", idx == 1 and "->" or "   or", package_in_group:displayname(), package_in_group:version_str() or "", package.get_configs_str(package_in_group))
+                                cprint("  ${yellow}%s${clear} %s %s ${dim}%s", idx == 1 and "->" or "   or",
+                                    package_in_group:displayname(), package_in_group:version_str() or "",
+                                    package.get_configs_str(package_in_group))
+                                for _, tip in ipairs(package_in_group:get("installtips")) do
+                                    if idx == 1 then
+                                        cprint("     ${yellow}*${clear} %s", tip)
+                                    else
+                                        cprint("        ${yellow}*${clear} %s", tip)
+                                    end
+                                end
                                 packages_showed[tostring(package_in_group)] = true
                             end
                             packages_group[group] = nil
                         else
-                            cprint("  ${yellow}->${clear} %s %s ${dim}%s", instance:displayname(), instance:version_str() or "", package.get_configs_str(instance))
+                            cprint("  ${yellow}->${clear} %s %s ${dim}%s",
+                                instance:displayname(), instance:version_str() or "",
+                                package.get_configs_str(instance))
+                            for _, tip in ipairs(instance:get("installtips")) do
+                                cprint("     ${yellow}*${clear} %s", tip)
+                            end
                             packages_showed[tostring(instance)] = true
                         end
                     end
@@ -379,8 +398,8 @@ function _should_install_package(instance)
     return result
 end
 
--- install packages
-function _install_packages(packages_install, packages_download, installdeps)
+-- do install packages
+function _do_install_packages(packages_install, packages_download, installdeps)
 
     -- we need to hide wait characters if is not a tty
     local show_wait = io.isatty()
@@ -663,9 +682,7 @@ function _get_package_installdeps(packages)
 end
 
 -- install packages
-function main(requires, opt)
-
-    -- init options
+function _install_packages(requires, opt)
     opt = opt or {}
 
     -- load packages
@@ -728,16 +745,31 @@ function main(requires, opt)
     if #packages_unknown > 0 then
         cprint("${bright color.warning}note: ${clear}the following packages were not found in any repository (check if they are spelled correctly):")
         for _, instance in ipairs(packages_unknown) do
-            print("  -> %s", instance:displayname())
+            local tips
+            local possible_package = package.get_possible_package(instance:name())
+            if possible_package then
+                tips = string.format(", maybe ${bright}%s %s${clear} in %s", possible_package.name, possible_package.version, possible_package.reponame)
+            end
+            cprint("  -> %s%s", instance:displayname(), tips or "")
         end
         has_errors = true
     end
 
     -- exists unsupported packages?
     if #packages_unsupported > 0 then
-        cprint("${bright color.warning}note: ${clear}the following packages are unsupported on $(plat)/$(arch):")
+        local packages_unsupported_maps = {}
         for _, instance in ipairs(packages_unsupported) do
-            print("  -> %s %s", instance:displayname(), instance:version_str() or "")
+            local key = instance:plat() .. "/" .. instance:arch()
+            packages_unsupported_maps[key] = packages_unsupported_maps[key] or {}
+            table.insert(packages_unsupported_maps[key], instance)
+        end
+        for key, instances in pairs(packages_unsupported_maps) do
+            cprint("${bright color.warning}note: ${clear}the following packages are unsupported on %s,", key)
+            cprint("${clear}you can try running ${bright}`xrepo update-repo`${clear} to update repositories or open a pr to improve these packages:")
+            for _, instance in ipairs(instances) do
+                cprint("  ${yellow}->${clear} %s %s ${dim}%s",
+                    instance:displayname(), instance:version_str() or "", package.get_configs_str(instance))
+            end
         end
         has_errors = true
     end
@@ -760,7 +792,7 @@ function main(requires, opt)
     end
 
     -- get user confirm
-    local confirm, packages_modified = _get_confirm(packages_install)
+    local confirm, packages_modified = _get_confirm(packages_install, opt)
     if not confirm then
         local packages_must = {}
         for _, instance in ipairs(packages_install) do
@@ -794,7 +826,7 @@ function main(requires, opt)
     _sort_packages_urls(packages_download)
 
     -- install all required packages from repositories
-    _install_packages(packages_install, packages_download, installdeps)
+    _do_install_packages(packages_install, packages_download, installdeps)
 
     -- disable other packages in same group
     _disable_other_packages_in_group(packages)
@@ -807,6 +839,18 @@ function main(requires, opt)
     if option.get("upgrade") then
         _show_upgraded_packages(packages)
     end
+    return packages
+end
+
+function main(requires, opt)
+    -- we need to install toolchain packages first,
+    -- because we will call compiler-specific api in package.on_load,
+    -- and we will check package toolchains before calling package.on_load
+    --
+    -- @see https://github.com/xmake-io/xmake/pull/5466
+    local packages = {}
+    table.join2(packages, _install_packages(requires, table.join(opt or {}, {toolchain = true})))
+    table.join2(packages, _install_packages(requires, opt))
 
     -- lock packages
     lock_packages(packages)
