@@ -12,7 +12,7 @@
 -- See the License for the specific language governing permissions and
 -- limitations under the License.
 --
--- Copyright (C) 2015-present, TBOOX Open Source Group.
+-- Copyright (C) 2015-present, Xmake Open Source Community.
 --
 -- @author      ruki
 -- @file        autoconf.lua
@@ -44,7 +44,7 @@ end
 
 -- get build directory
 function _get_buildir()
-    return config.buildir() or "build"
+    return config.builddir() or "build"
 end
 
 -- get artifacts directory
