@@ -12,7 +12,7 @@
 -- See the License for the specific language governing permissions and
 -- limitations under the License.
 --
--- Copyright (C) 2015-present, TBOOX Open Source Group.
+-- Copyright (C) 2015-present, Xmake Open Source Community.
 --
 -- @author      ruki
 -- @file        archive.lua
@@ -22,8 +22,15 @@
 import("core.base.option")
 import("lib.detect.find_file")
 import("lib.detect.find_tool")
+import("archive_xmz")
 import("extension", {alias = "get_archive_extension"})
 
+-- archive archivefile using xmake compress module
+function _archive_using_xmz(archivefile, inputfiles, extension, opt)
+    archive_xmz(archivefile, inputfiles, opt)
+    return true
+end
+
 -- archive archivefile using zip
 function _archive_using_zip(archivefile, inputfiles, extension, opt)
 
@@ -299,7 +306,7 @@ function _archive(archivefile, inputfiles, extension, archivers, opt)
             return true
         end
     end
-    raise("cannot archive %s, %s!", path.filename(archivefile), errors or "archivers not found!")
+    raise("cannot archive %s, %s!", path.filename(archivefile), errors or "no archiver(like zip, ...) found")
 end
 
 -- only archive tar file
@@ -312,7 +319,7 @@ function _archive_tarfile(archivefile, tarfile, opt)
     return _archive(archivefile, tarfile, extension, archivers[extension], opt)
 end
 
--- archive archive file
+-- archive file
 --
 -- @param archivefile   the archive file. e.g. *.tar.gz, *.zip, *.7z, *.tar.bz2, ..
 -- @param inputfiles    the input file or directory or list
@@ -334,6 +341,7 @@ function main(archivefile, inputfiles, opt)
     ,   [".tar"]        = {_archive_using_tar}
     ,   [".tar.gz"]     = {_archive_using_tar, _archive_using_gzip}
     ,   [".tar.xz"]     = {_archive_using_tar, _archive_using_xz}
+    ,   [".xmz"]        = {_archive_using_xmz}
     }
 
     -- get extension
