@@ -12,7 +12,7 @@
 -- See the License for the specific language governing permissions and
 -- limitations under the License.
 --
--- Copyright (C) 2015-present, TBOOX Open Source Group.
+-- Copyright (C) 2015-present, Xmake Open Source Community.
 --
 -- @author      ruki
 -- @file        format.lua
@@ -40,7 +40,7 @@ task("format")
                                                   "e.g.",
                                                   "    - xmake format --files=src/main.c",
                                                   "    - xmake format --files='src/*.c' [target]",
-                                                  "    - xmake format --files='src/**.c|excluded_file.c",
+                                                  "    - xmake format --files='src/**.c|excluded_file.c'",
                                                   "    - xmake format --files='src/main.c" .. path.envsep() .. "src/test.c'" },
                                                   {},
                     {nil, "target",  "v",  nil,   "The target name. It will format all default targets if this parameter is not specified."
