@@ -12,7 +12,7 @@
 -- See the License for the specific language governing permissions and
 -- limitations under the License.
 --
--- Copyright (C) 2015-present, TBOOX Open Source Group.
+-- Copyright (C) 2015-present, Xmake Open Source Community.
 --
 -- @author      ruki
 -- @file        cmakelists.lua
@@ -29,10 +29,10 @@ import("core.project.rule")
 import("core.platform.platform")
 import("lib.detect.find_tool")
 import("private.utils.batchcmds")
-import("private.utils.rule_groups")
 import("private.utils.target", {alias = "target_utils"})
 import("plugins.project.utils.target_cmds", {rootdir = os.programdir()})
-import("rules.c++.modules.modules_support.compiler_support", {alias = "module_compiler_support", rootdir = os.programdir()})
+import("rules.c++.modules.support", {alias = "module_support", rootdir = os.programdir()})
+import("private.utils.executable_path")
 
 -- get cmake version
 function _get_cmake_version()
@@ -50,7 +50,7 @@ end
 -- has c++ modules sources
 function _has_cxxmodules_sources()
     for _, target in ipairs(project.ordertargets()) do
-        if module_compiler_support.contains_modules(target) then
+        if module_support.contains_modules(target) then
             return true
         end
     end
@@ -303,7 +303,7 @@ function _set_compiler(cmakelists)
     if config.get("toolchain") then
         local cc = platform.tool("cc")
         if cc then
-            cc = path.unix(cc)
+            cc = path.unix(executable_path(cc))
             cmakelists:print("set(CMAKE_C_COMPILER \"%s\")", cc)
         end
         local cxx, cxx_name = platform.tool("cxx")
@@ -321,7 +321,7 @@ function _set_compiler(cmakelists)
                     cxx = name
                 end
             end
-            cxx = path.unix(cxx)
+            cxx = path.unix(executable_path(cxx))
             cmakelists:print("set(CMAKE_CXX_COMPILER \"%s\")", cxx)
         end
     end
@@ -349,26 +349,32 @@ function _add_project(cmakelists, outputdir)
     -- set project name
     local project_name = project.name()
     if not project_name then
-        for _, target in table.orderpairs(project.targets()) do
+        local project_targets = target_utils.get_project_targets()
+        for _, target in table.orderpairs(project_targets) do
             project_name = target:name()
-            break
         end
     end
+    if _can_native_support_for_cxxmodules() then
+        cmakelists:print("set(CMAKE_CXX_SCAN_FOR_MODULES ON)")
+    end
+    local languages = _get_project_languages()
     if project_name then
         local project_info = ""
         local project_version = project.version()
         if project_version then
             project_info = project_info .. " VERSION " .. project_version
         end
-        local languages = _get_project_languages()
         if languages then
             cmakelists:print("project(%s%s LANGUAGES %s)", project_name, project_info, table.concat(languages, " "))
         else
             cmakelists:print("project(%s%s)", project_name, project_info)
         end
     end
-    if _can_native_support_for_cxxmodules() then
-        cmakelists:print("set(CMAKE_CXX_SCAN_FOR_MODULES ON)")
+    -- Define a language-independant global compiler_id variable
+    if (languages and #languages > 0) then
+        cmakelists:print("set(CURRENT_COMPILER_ID ${CMAKE_%s_COMPILER_ID})", _get_project_languages()[1])
+    else
+        cmakelists:print("set(CURRENT_COMPILER_ID ${CMAKE_C_COMPILER_ID})") -- C should be defined by default if not specified
     end
     cmakelists:print("")
 end
@@ -454,25 +460,82 @@ function _add_target_dependencies(cmakelists, target)
     end
 end
 
+function _print_target_sources(cmakelists, target, files, visibility, opt)
+    opt = opt or {}
+    local cmake_version = _get_cmake_version()
+    local has_fileset_support = cmake_version and cmake_version:ge("3.23")
+    local fileset = ""
+    if has_fileset_support and opt.set then
+        fileset = "FILE_SET " .. opt.set .. " FILES"
+    end
+
+    cmakelists:print("target_sources(%s %s %s", target:name(), visibility, fileset)
+    for _, file in ipairs(files) do
+        cmakelists:print("    " .. file)
+    end
+    cmakelists:print(")")
+end
+
 -- add target sources
 function _add_target_sources(cmakelists, target, outputdir)
     local has_cuda = false
-    cmakelists:print("target_sources(%s PRIVATE", target:name())
     local sourcebatches = target:sourcebatches()
     for name, sourcebatch in table.orderpairs(sourcebatches) do
+        local public_sources
+        local private_sources
         if _sourcebatch_is_built(sourcebatch) then
+            local module_sourcebatch = false
             for _, sourcefile in ipairs(sourcebatch.sourcefiles) do
-                cmakelists:print("    " .. _get_relative_unix_path(sourcefile, outputdir))
+                if _has_cxxmodules_sources() and name == "c++.build.modules" then
+                    module_sourcebatch = true
+                    local fileconfig = target:fileconfig(sourcefile)
+                    if fileconfig and fileconfig.public then
+                        public_sources = public_sources or {}
+                        table.insert(public_sources, _get_relative_unix_path(sourcefile, outputdir))
+                    else
+                        private_sources = private_sources or {}
+                        table.insert(private_sources, _get_relative_unix_path(sourcefile, outputdir))
+                    end
+                else
+                    private_sources = private_sources or {}
+                    table.insert(private_sources, _get_relative_unix_path(sourcefile, outputdir))
+                end
+            end
+            if public_sources then
+                cmakelists:print(format("# public sourcefiles from sourcebatch %s for target %s", name, target:fullname()))
+                _print_target_sources(cmakelists, target, public_sources, "PUBLIC", {set = module_sourcebatch and "CXX_MODULES"})
+            end
+            if private_sources then
+                cmakelists:print(format("# private sourcefiles from sourcebatch %s for target %s", name, target:fullname()))
+                _print_target_sources(cmakelists, target, private_sources, "PRIVATE", {set = module_sourcebatch and "CXX_MODULES"})
             end
         end
         if sourcebatch.sourcekind == "cu" then
             has_cuda = true
         end
     end
-    for _, headerfile in ipairs(target:headerfiles()) do
-        cmakelists:print("    " .. _get_relative_unix_path(headerfile, outputdir))
+    if target:headerfiles() then
+        local public_headers
+        local private_headers
+        for _, headerfile in ipairs(target:headerfiles()) do
+            local fileconfig = target:fileconfig(headerfile)
+            if fileconfig and fileconfig.public then
+                public_headers = public_headers or {}
+                table.insert(public_headers, _get_relative_unix_path(headerfile, outputdir))
+            else
+                private_headers = private_headers or {}
+                table.insert(private_headers, _get_relative_unix_path(headerfile, outputdir))
+            end
+        end
+        if public_headers then
+            cmakelists:print(format("# public headers for target %s", target:fullname()))
+            _print_target_sources(cmakelists, target, public_headers, "PUBLIC", {set = "HEADERS"})
+        end
+        if private_headers then
+            cmakelists:print(format("# private headers for target %s", target:fullname()))
+            _print_target_sources(cmakelists, target, private_headers, "PRIVATE", {set = "HEADERS"})
+        end
     end
-    cmakelists:print(")")
     if has_cuda then
         cmakelists:print("set_target_properties(%s PROPERTIES CUDA_SEPARABLE_COMPILATION ON)", target:name())
         local devlink = target:policy("build.cuda.devlink") or target:values("cuda.build.devlink")
@@ -722,15 +785,15 @@ function _add_target_compile_options(cmakelists, target, outputdir)
     local compilernames = {
         clang = "Clang",
         clangxx = "Clang",
-        gcc = "Gcc",
-        gxx = "Gcc",
+        gcc = "GNU",
+        gxx = "GNU",
         cl = "MSVC",
         link = "MSVC"
     }
     for _, toolname in toolnames:keys() do
         local name = compilernames[toolname]
         if name then
-            cmakelists:print("if(%s)", name)
+            cmakelists:print("if(CURRENT_COMPILER_ID STREQUAL \"%s\")", name)
             _add_target_compile_options_for_compiler(toolname)
             cmakelists:print("endif()")
         end
@@ -757,17 +820,17 @@ function _add_target_values(cmakelists, target, name)
         if name:endswith("s") then
             name = name:sub(1, #name - 1)
         end
-        cmakelists:print("if(MSVC)")
+        cmakelists:print("if(CURRENT_COMPILER_ID STREQUAL \"MSVC\")")
         local flags_cl = _map_compflags("cl", "c", name, values)
         for _, flag in ipairs(flags_cl) do
             cmakelists:print("    target_compile_options(%s PRIVATE %s)", target:name(), flag)
         end
-        cmakelists:print("elseif(Clang)")
+        cmakelists:print("elseif(CURRENT_COMPILER_ID STREQUAL \"Clang\")")
         local flags_clang = _map_compflags("clang", "c", name, values)
         for _, flag in ipairs(flags_clang) do
             cmakelists:print("    target_compile_options(%s PRIVATE %s)", target:name(), flag)
         end
-        cmakelists:print("elseif(Gcc)")
+        cmakelists:print("elseif(CURRENT_COMPILER_ID STREQUAL \"GNU\")")
         local flags_gcc = _map_compflags("gcc", "c", name, values)
         for _, flag in ipairs(flags_gcc) do
             cmakelists:print("    target_compile_options(%s PRIVATE %s)", target:name(), flag)
@@ -834,7 +897,7 @@ function _add_target_languages(cmakelists, target)
                     if flag:endswith('++') then
                         cmakelists:print('foreach(standard 26 23 20 17 14 11 98)')
                         cmakelists:print('    include(CheckCXXCompilerFlag)')
-                        cmakelists:print('    if(MSVC)')
+                        cmakelists:print('    if(CURRENT_COMPILER_ID STREQUAL \"MSVC\")')
                         cmakelists:print('        check_cxx_compiler_flag("/std:%s${standard}" %s_support_%s_standard_${standard})', flag, target:name(), flag)
                         cmakelists:print('    else()')
                         cmakelists:print('        check_cxx_compiler_flag("-std=%s${standard}" %s_support_%s_standard_${standard})', flag, target:name(), flag)
@@ -847,7 +910,7 @@ function _add_target_languages(cmakelists, target)
                     else
                         cmakelists:print('foreach(standard 23 17 11 99 90)')
                         cmakelists:print('    include(CheckCCompilerFlag)')
-                        cmakelists:print('    if(MSVC)')
+                        cmakelists:print('    if(CURRENT_COMPILER_ID STREQUAL \"MSVC\")')
                         cmakelists:print('        check_c_compiler_flag("/std:%s${standard}" %s_support_%s_standard_${standard})', flag, target:name(), flag)
                         cmakelists:print('    else()')
                         cmakelists:print('        check_c_compiler_flag("-std=%s${standard}" %s_support_%s_standard_${standard})', flag, target:name(), flag)
@@ -887,7 +950,7 @@ function _add_target_optimization(cmakelists, target)
     }
     local optimization = target:get("optimize")
     if optimization then
-        cmakelists:print("if(MSVC)")
+        cmakelists:print("if(CURRENT_COMPILER_ID STREQUAL \"MSVC\")")
         cmakelists:print("    target_compile_options(%s PRIVATE %s)", target:name(), flags_msvc[optimization])
         cmakelists:print("else()")
         cmakelists:print("    target_compile_options(%s PRIVATE %s)", target:name(), flags_gcc[optimization])
@@ -915,7 +978,7 @@ function _add_target_symbols(cmakelists, target)
         if levels:has("hidden") then
             table.insert(flags_gcc, "-fvisibility=hidden")
         end
-        cmakelists:print("if(MSVC)")
+        cmakelists:print("if(CURRENT_COMPILER_ID STREQUAL \"MSVC\")")
         if #flags_msvc > 0 then
             cmakelists:print("    target_compile_options(%s PRIVATE %s)", target:name(), table.concat(flags_msvc, " "))
         end
@@ -936,7 +999,7 @@ function _add_target_runtimes(cmakelists, target)
     local cmake_minver = _get_cmake_minver()
     if cmake_minver:ge("3.15.0") then
         local runtimes = target:get("runtimes")
-        cmakelists:print("if(MSVC)")
+        cmakelists:print("if(CURRENT_COMPILER_ID STREQUAL \"MSVC\")")
         if runtimes then
             if runtimes == "MT" then
                 runtimes = "MultiThreaded"
@@ -1009,9 +1072,12 @@ function _add_target_link_libraries(cmakelists, target, outputdir)
         end
     end
 
+
     local has_links = #target:objectfiles() > objectfiles_set:size()
+    local key = target:name() .. "_" .. hash.rand64()
     if has_links then
-        cmakelists:print("target_link_libraries(%s PRIVATE", target:name())
+        cmakelists:print("add_library(target_objectfiles_%s OBJECT IMPORTED GLOBAL)", key)
+        cmakelists:print("set_property(TARGET target_objectfiles_%s PROPERTY IMPORTED_OBJECTS", key)
         for _, objectfile in ipairs(target:objectfiles()) do
             if not objectfiles_set:has(objectfile) then
                 cmakelists:print("    " .. _get_relative_unix_path_to_cmake(objectfile, outputdir))
@@ -1021,7 +1087,8 @@ function _add_target_link_libraries(cmakelists, target, outputdir)
 
     if #object_deps ~= 0 then
         if not has_links then
-            cmakelists:print("target_link_libraries(%s PRIVATE", target:name())
+            cmakelists:print("add_library(target_objectfiles_%s OBJECT IMPORTED GLOBAL)", key)
+            cmakelists:print("set_property(TARGET target_objectfiles_%s PROPERTY IMPORTED_OBJECTS", key)
             has_links = true
         end
         for _, dep in ipairs(object_deps) do
@@ -1031,6 +1098,7 @@ function _add_target_link_libraries(cmakelists, target, outputdir)
 
     if has_links then
         cmakelists:print(")")
+        cmakelists:print("target_link_libraries(%s PRIVATE target_objectfiles_%s)", target:name(), key)
     end
 end
 
@@ -1046,7 +1114,7 @@ function _add_target_link_directories(cmakelists, target, outputdir)
             end
             cmakelists:print(")")
         else
-            cmakelists:print("if(MSVC)")
+            cmakelists:print("if(CURRENT_COMPILER_ID STREQUAL \"MSVC\")")
             cmakelists:print("    target_link_libraries(%s PRIVATE", target:name())
             for _, linkdir in ipairs(linkdirs) do
                 cmakelists:print("        -libpath:" .. _get_relative_unix_path(linkdir, outputdir))
@@ -1105,15 +1173,15 @@ function _add_target_link_options(cmakelists, target, outputdir)
     local linkernames = {
         clang = "Clang",
         clangxx = "Clang",
-        gcc = "Gcc",
-        gxx = "Gcc",
+        gcc = "GNU",
+        gxx = "GNU",
         cl = "MSVC",
         link = "MSVC"
     }
     for _, toolname in toolnames:keys() do
         local name = linkernames[toolname]
         if name then
-            cmakelists:print("if(%s)", name)
+            cmakelists:print("if(CURRENT_COMPILER_ID STREQUAL \"%s\")", name)
             _add_target_link_options_for_linker(toolname)
             cmakelists:print("endif()")
         end
@@ -1155,7 +1223,13 @@ function _get_command_string(cmd, outputdir)
     elseif kind == "mkdir" then
         return string.format("${CMAKE_COMMAND} -E make_directory %s", _get_relative_unix_path_to_cmake(cmd.dir, outputdir))
     elseif kind == "show" then
-        return string.format("echo %s", colors.ignore(cmd.showtext))
+        local text = colors.ignore(cmd.showtext)
+        -- we need to translate paths in text
+        -- https://github.com/xmake-io/xmake/issues/6553
+        if is_host("windows") then
+            text = path.unix(text)
+        end
+        return string.format("echo %s", text)
     end
 end
 
@@ -1170,7 +1244,7 @@ function _add_target_custom_commands_for_batchcmds(cmakelists, target, outputdir
         --
         -- @see https://gitlab.kitware.com/cmake/cmake/-/issues/17802
         --
-        local key = target:name() .. "_" .. hash.uuid():split("-", {plain = true})[1]
+        local key = target:name() .. "_" .. hash.rand64()
         cmakelists:print("add_custom_command(OUTPUT output_%s", key)
         for _, cmd in ipairs(cmds) do
             local command = _get_command_string(cmd, outputdir)
@@ -1201,9 +1275,6 @@ end
 -- add target custom commands
 function _add_target_custom_commands(cmakelists, target, outputdir)
 
-    -- build sourcebatch groups first
-    local sourcegroups = rule_groups.build_sourcebatch_groups(target, target:sourcebatches())
-
     -- ignore c++ modules rules
     local ignored_rules
     if _can_native_support_for_cxxmodules() then
@@ -1212,17 +1283,12 @@ function _add_target_custom_commands(cmakelists, target, outputdir)
 
     -- add before commands
     -- we use irpairs(groups), because the last group that should be given the highest priority.
-    local cmds_before = {}
-    target_cmds.get_target_buildcmd(target, cmds_before, {suffix = "before", ignored_rules = ignored_rules})
-    target_cmds.get_target_buildcmd_sourcegroups(target, cmds_before, sourcegroups, {suffix = "before", ignored_rules = ignored_rules})
     -- rule.on_buildcmd_files should also be executed before building the target, as cmake PRE_BUILD does not work.
-    target_cmds.get_target_buildcmd_sourcegroups(target, cmds_before, sourcegroups, {ignored_rules = ignored_rules})
+    local cmds_before = target_cmds.get_target_buildcmds(target, {ignored_rules = ignored_rules, stages = {"before", "on"}})
     _add_target_custom_commands_for_batchcmds(cmakelists, target, outputdir, "before", cmds_before)
 
     -- add after commands
-    local cmds_after = {}
-    target_cmds.get_target_buildcmd_sourcegroups(target, cmds_after, sourcegroups, {suffix = "after", ignored_rules = ignored_rules})
-    target_cmds.get_target_buildcmd(target, cmds_after, {suffix = "after", ignored_rules = ignored_rules})
+    local cmds_after = target_cmds.get_target_buildcmds(target, {ignored_rules = ignored_rules, stages = {"after"}})
     _add_target_custom_commands_for_batchcmds(cmakelists, target, outputdir, "after", cmds_after)
 end
 
@@ -1326,26 +1392,17 @@ function _generate_cmakelists(cmakelists, outputdir)
     _add_project(cmakelists, outputdir)
 
     -- add targets
-    for _, target in table.orderpairs(project.targets()) do
+    local project_targets = target_utils.get_project_targets()
+    for _, target in table.orderpairs(project_targets) do
         _add_target(cmakelists, target, outputdir)
     end
 end
 
--- make
 function make(outputdir)
-
-    -- enter project directory
     local oldir = os.cd(os.projectdir())
-
-    -- open the cmakelists
     local cmakelists = io.open(path.join(outputdir, "CMakeLists.txt"), "w")
-
-    -- generate cmakelists
+    target_cmds.prepare_targets()
     _generate_cmakelists(cmakelists, outputdir)
-
-    -- close the cmakelists
     cmakelists:close()
-
-    -- leave project directory
     os.cd(oldir)
 end
