@@ -12,7 +12,7 @@
 -- See the License for the specific language governing permissions and
 -- limitations under the License.
 --
--- Copyright (C) 2015-present, TBOOX Open Source Group.
+-- Copyright (C) 2015-present, Xmake Open Source Community.
 --
 -- @author      ruki
 -- @file        xmake.lua
@@ -23,7 +23,7 @@ rule("go.build")
     add_deps("go.env")
     on_load(function (target)
         -- we disable to build across targets in parallel, because the source files may depend on other target modules
-        target:set("policy", "build.across_targets_in_parallel", false)
+        target:set("policy", "build.fence", true)
         -- xxx.a
         if target:is_static() then
             target:set("prefixname", "")
