@@ -12,7 +12,7 @@
 -- See the License for the specific language governing permissions and
 -- limitations under the License.
 --
--- Copyright (C) 2015-present, TBOOX Open Source Group.
+-- Copyright (C) 2015-present, Xmake Open Source Community.
 --
 -- @author      ruki
 -- @file        xmake.lua
@@ -85,8 +85,8 @@ rule("utils.hlsl2spv")
             outputfile = headerfile
 
             -- add commands
-            local argv = {"lua", "private.utils.bin2c", "--nozeroend", "-i", path(spvfilepath), "-o", path(headerfile)}
-            batchcmds:vrunv(os.programfile(), argv, {envs = {XMAKE_SKIP_HISTORY = "y"}})
+            local argv = {"--nozeroend", "-i", path(spvfilepath), "-o", path(headerfile)}
+            batchcmds:vlua("private.utils.bin2c", argv)
         end
 
         batchcmds:add_depfiles(sourcefile_hlsl)
