@@ -12,7 +12,7 @@
 -- See the License for the specific language governing permissions and
 -- limitations under the License.
 --
--- Copyright (C) 2015-present, TBOOX Open Source Group.
+-- Copyright (C) 2015-present, Xmake Open Source Community.
 --
 -- @author      ruki
 -- @file        inherit_links.lua
@@ -80,11 +80,25 @@ function main(target)
             end
         end
 
-        _add_export_value(target, "linkdirs", path.directory(targetfile))
+        local implibfile = target:artifactfile("implib")
+        if implibfile then
+            _add_export_value(target, "linkdirs", path.directory(implibfile))
+        else
+            _add_export_value(target, "linkdirs", path.directory(targetfile))
+        end
+
         if target:rule("go") then
             -- we need to add includedirs to support import modules for golang
             _add_export_value(target, "includedirs", path.directory(targetfile))
         end
+
+        if target:rule("rust") then
+            local cratetype = target:values("rust.cratetype")
+            -- only bin, lib, rlib, dylib can make use of --extern CRATE[=PATH]
+            if cratetype ~= "staticlib" and cratetype ~= "cdylib" then
+                _add_export_value(target, "frameworks", target:targetfile())
+            end
+        end
     end
 
     -- we export all links and linkdirs in self/packages/options to the parent target by default
